@@ -17,13 +17,5 @@ $noresave{$key} = "$nosave";
 # labels from external_latex_labels array.
 
 
-$key = q/Sec:para/;
-$external_latex_labels{$key} = q|4 Parallelism|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:Examples/;
-$external_latex_labels{$key} = q|3.1 Running examples|; 
-$noresave{$key} = "$nosave";
-
 1;
 
