@@ -636,9 +636,9 @@ SUBROUTINE cutoff_stres_sigmaewa( alpha, sdewald, sigmaewa )
      DO na = 1, nat
         arg = (g(1,ng) * tau(1,na) + g(2,ng) * tau(2,na) + &
                g(3,ng) * tau(3,na) ) * tpi
-        rhostar = rhostar + CMPLX(zv(ityp(na))) * CMPLX(COS(arg),SIN(arg),KIND=DP)
+        rhostar = rhostar + CMPLX(zv(ityp(na)),KIND=DP) * CMPLX(COS(arg),SIN(arg),KIND=DP)
      ENDDO
-     rhostar = rhostar / CMPLX(omega)
+     rhostar = rhostar / CMPLX(omega,KIND=DP)
      sewald = tpi * e2 * EXP(-g2a) / g2* cutoff_2D(ng) * ABS(rhostar)**2
      ! ... sewald is an other diagonal term that is similar to the diagonal terms 
      !     in the other stress contributions. It basically gives a term prop to 
