@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2022 Quantum ESPRESSO group
+! Copyright (C) 2001-2023 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -23,6 +23,7 @@ SUBROUTINE offset_atom_wfc( hubbard_only, lflag, offset, counter )
   USE ldaU,             ONLY : Hubbard_l, Hubbard_n, Hubbard_l2, Hubbard_n2,        &
                                Hubbard_l3, Hubbard_n3, is_hubbard, is_hubbard_back, &
                                backall, hubbard_occ
+  USE upf_utils,        ONLY : l_to_spdf, capital
   !
   IMPLICIT NONE
   !
@@ -39,8 +40,6 @@ SUBROUTINE offset_atom_wfc( hubbard_only, lflag, offset, counter )
   CHARACTER(LEN=2) :: label_aux
   CHARACTER(LEN=2), ALLOCATABLE :: label(:)
   CHARACTER(LEN=6), EXTERNAL :: int_to_char
-  CHARACTER(LEN=1), EXTERNAL :: l_to_spdf
-  CHARACTER(LEN=1), EXTERNAL :: capital
   !
   counter = 0
   offset(:) = -1
@@ -62,14 +61,14 @@ SUBROUTINE offset_atom_wfc( hubbard_only, lflag, offset, counter )
      !
      IF (is_hubbard(nt)) &
            label_hub = TRIM(int_to_char(Hubbard_n(nt))) // &
-                            l_to_spdf(Hubbard_l(nt),.TRUE.)
+                            l_to_spdf(Hubbard_l(nt))
      !
      IF (is_hubbard_back(nt)) THEN
            label_hub2 = TRIM(int_to_char(Hubbard_n2(nt))) // &
-                             l_to_spdf(Hubbard_l2(nt),.TRUE.)
+                             l_to_spdf(Hubbard_l2(nt))
         IF (backall(nt)) &
            label_hub3 = TRIM(int_to_char(Hubbard_n3(nt))) // &
-                             l_to_spdf(Hubbard_l3(nt),.TRUE.)
+                             l_to_spdf(Hubbard_l3(nt))
      ENDIF
      !
      ALLOCATE(label(ldim))
@@ -94,6 +93,14 @@ SUBROUTINE offset_atom_wfc( hubbard_only, lflag, offset, counter )
            !
            l = upf(nt)%lchi(n)
            !
+           ! Note: For FR-PPs, the j = l-1/2 is full, the j = l+1/2 is empty.
+           ! For example, for Eu-4f one 4f orbital has non-zero occupancy, while 
+           ! other 4f orbital is empty. However, hubbard_occ is defined as a sum
+           ! of both occupied and empty 4f orbitals (see determine_hubbard_occ).
+           ! Therefore, both occupied and empty 4f states will be considered in 
+           ! the calculation (the full 4f shell), which is consistent with the 
+           ! NR/SR case where the full 4f shell is considered. 
+           !
            IF (is_hubbard(nt)) THEN
               IF (label(n)==label_hub) THEN
                  IF (hubbard_occ(nt,1) > 0.D0) THEN
@@ -102,15 +109,6 @@ SUBROUTINE offset_atom_wfc( hubbard_only, lflag, offset, counter )
                     CALL errore('offset_atom_wfc', 'Hubbard manifold with &
                             &zero occupations is not allowed',1) 
                  ENDIF
-                 ! For FR-PPs, hubbard_occ cannot be used here because it does not
-                 ! allow to distinguish between the occupied and unoccupied channels.
-                 IF (noncolin .AND. upf(nt)%has_so) THEN
-                    IF (upf(nt)%oc(n) > 0.D0) THEN
-                       hubbard_wfc = .TRUE.
-                    ELSE
-                       hubbard_wfc = .FALSE.
-                    ENDIF
-                 ENDIF
               ENDIF
            ENDIF
            !
