@@ -97,6 +97,8 @@ SUBROUTINE setup()
   USE additional_kpoints, ONLY : add_additional_kpoints
   USE control_flags,      ONLY : sic
   USE sic_mod,            ONLY : init_sic, occ_f2fn, sic_energy
+  USE random_numbers,     ONLY : set_random_seed
+  USE dynamics_module,    ONLY : control_temp
   !
   IMPLICIT NONE
   !
@@ -675,14 +677,18 @@ SUBROUTINE setup()
   ! ... checks and initializations to be performed after parallelization setup
   !
   IF ( lberry .OR. lelfield .OR. lorbm ) THEN
-     IF ( npool > 1 ) CALL errore( 'iosys', &
+     IF ( npool > 1 ) CALL errore( 'setup', &
           'Berry Phase/electric fields not implemented with pools', 1 )
   END IF
+  IF ( gamma_only .AND. nkstot == 1 .AND. npool > 1 ) CALL errore( 'setup', &
+          'Gamma-only calculations not allowed with pools', 1 )
   IF ( xclib_dft_is('hybrid') ) THEN
      IF ( nks == 0 ) CALL errore('setup','pools with no k-points' &
           & // ' not allowed for hybrid functionals',1)
      IF ( tstress .and. npool > 1 )  CALL errore('setup', &
          'stress for hybrid functionals not available with pools', 1)
+     !!!IF ( tstress )  CALL errore('setup', &
+     !!!    'stress for hybrid functionals not available', 1)
      !
      CALL setup_exx  ()
      !
@@ -699,7 +705,10 @@ SUBROUTINE setup()
      IF (sic_energy) CALL occ_f2fn()
   END IF
   !
-  RETURN
+  ! ... next command prevents different MD runs to start
+  ! ... with exactly the same "random" velocities
+  !
+  IF (lmd.AND.control_temp) CALL set_random_seed( )
   !
 END SUBROUTINE setup
 !
