@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2015 Quantum ESPRESSO group
+! Copyright (C) 2001-2023 Quantum ESPRESSO Foundation
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -45,12 +45,10 @@ SUBROUTINE sum_band_gpu()
   USE xc_lib,               ONLY : xclib_dft_is
   USE paw_symmetry,         ONLY : PAW_symmetrize
   USE paw_variables,        ONLY : okpaw
-  USE becmod,               ONLY : allocate_bec_type, deallocate_bec_type, &
+  USE becmod,               ONLY : allocate_bec_type_acc, deallocate_bec_type_acc, &
                                    becp
   USE gcscf_module,         ONLY : lgcscf, gcscf_calc_nelec
-  USE wavefunctions_gpum,   ONLY : evc_d, using_evc, using_evc_d
   USE wvfct_gpum,           ONLY : using_et
-  USE becmod_subs_gpum,     ONLY : using_becp_auto
   !
   IMPLICIT NONE
   !
@@ -135,8 +133,7 @@ SUBROUTINE sum_band_gpu()
   !
   ! ... Allocate (and later deallocate) arrays needed in specific cases
   !
-  IF ( okvan ) CALL allocate_bec_type( nkb, this_bgrp_nbnd, becp, intra_bgrp_comm )
-  IF ( okvan ) CALL using_becp_auto( 2 )
+  IF ( okvan ) CALL allocate_bec_type_acc( nkb, this_bgrp_nbnd, becp, intra_bgrp_comm )
   IF (xclib_dft_is('meta') .OR. lxdm) ALLOCATE( kplusg(npwx), kplusg_evc(npwx,2) )
   !
   ! ... specialized routines are called to sum at Gamma or for each k point 
@@ -178,8 +175,7 @@ SUBROUTINE sum_band_gpu()
   IF (xclib_dft_is('meta') .OR. lxdm) THEN
     DEALLOCATE( kplusg, kplusg_evc )
   ENDIF
-  IF ( okvan ) CALL deallocate_bec_type ( becp )
-  IF ( okvan ) CALL using_becp_auto(2)
+  IF ( okvan ) CALL deallocate_bec_type_acc ( becp )
   !
   ! ... sum charge density over pools (distributed k-points) and bands
   !
@@ -299,7 +295,7 @@ SUBROUTINE sum_band_gpu()
        attributes(pinned) :: tg_rho_h
 #endif
        !
-       CALL using_evc_d(0); CALL using_et(0)
+       CALL using_et(0)
        !
        ! ... here we sum for each k point the contribution
        ! ... of the wavefunctions to the charge
@@ -334,8 +330,6 @@ SUBROUTINE sum_band_gpu()
           CALL start_clock_gpu( 'sum_band:buffer' )
           IF ( nks > 1 ) &
              CALL get_buffer ( evc, nwordwfc, iunwfc, ik )
-          IF ( nks > 1 ) CALL using_evc(2) ! get_buffer(evc, ...) evc is updated (intent out)
-          IF ( nks > 1 ) CALL using_evc_d(0) ! sync on the GPU
           !$acc update device(evc)
           !
           CALL stop_clock_gpu( 'sum_band:buffer' )
@@ -427,14 +421,12 @@ SUBROUTINE sum_band_gpu()
              ENDIF
              !
              IF (xclib_dft_is('meta') .OR. lxdm) THEN
-                !
-                CALL using_evc(0)
                 !
                 DO j = 1, 3
                    DO i = 1, npw
                      kplusgi = (xk(j,ik)+g(j,i)) * tpiba
-                     kplusg_evc(i,1) = CMPLX(0.D0,kplusgi) * evc(i,ibnd)
-                     IF ( ibnd < ibnd_end ) kplusg_evc(i,2) = CMPLX(0.d0,kplusgi) * evc(i,ibnd+1)
+                     kplusg_evc(i,1) = CMPLX(0._DP,kplusgi,KIND=DP) * evc(i,ibnd)
+                     IF ( ibnd < ibnd_end ) kplusg_evc(i,2) = CMPLX(0._DP,kplusgi,KIND=DP) * evc(i,ibnd+1)
                    ENDDO
                    !
                    ebnd = ibnd
@@ -536,7 +528,7 @@ SUBROUTINE sum_band_gpu()
        attributes(pinned) :: tg_rho_h, tg_rho_nc_h
 #endif
        !
-       CALL using_evc(0); CALL using_evc_d(0); CALL using_et(0)
+       CALL using_et(0)
        !
        ! ... here we sum for each k point the contribution
        ! ... of the wavefunctions to the charge
@@ -594,8 +586,6 @@ SUBROUTINE sum_band_gpu()
           CALL start_clock_gpu( 'sum_band:buffer' )
           IF ( nks > 1 ) THEN
              CALL get_buffer( evc, nwordwfc, iunwfc, ik )
-             CALL using_evc(2)
-             CALL using_evc_d(0)  ! sync evc on GPU, OPTIMIZE (use async here)
           ENDIF
           !$acc update device(evc)
           CALL stop_clock_gpu( 'sum_band:buffer' )
@@ -760,7 +750,7 @@ SUBROUTINE sum_band_gpu()
                    DO j=1,3
                       DO i = 1, npw
                         kplusgi = (xk(j,ik)+g(j,igk_k(i,ik))) * tpiba
-                        kplusg_evc(i,1) = CMPLX(0.D0,kplusgi,kind=DP) * evc(i,ibnd)
+                        kplusg_evc(i,1) = CMPLX(0._DP,kplusgi,KIND=DP) * evc(i,ibnd)
                       ENDDO
                       !
                       CALL wave_g2r( kplusg_evc(1:npw,1:1), psic, dffts, igk=igk_k(:,ik) )
@@ -963,8 +953,8 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
 #define cublasDgemm dgemm
 #endif
   USE kinds,              ONLY : DP
-  USE becmod,             ONLY : becp, calbec, allocate_bec_type
-  USE control_flags,      ONLY : gamma_only, tqr
+  USE becmod,             ONLY : becp, calbec
+  USE control_flags,      ONLY : gamma_only, tqr, offload_type 
   USE ions_base,          ONLY : nat, ntyp => nsp, ityp
   USE uspp,               ONLY : nkb, becsum, ebecsum, ofsbeta, &
                                  becsum_d, ebecsum_d, ofsbeta_d, vkb
@@ -979,10 +969,8 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
   USE us_exx,             ONLY : store_becxx0
   USE mp_bands,           ONLY : nbgrp,inter_bgrp_comm
   USE mp,                 ONLY : mp_sum
-  USE wavefunctions_gpum, ONLY : evc_d, using_evc, using_evc_d
   USE wvfct_gpum,         ONLY : et_d, wg_d, using_et, using_et_d, using_wg_d
-  USE becmod_subs_gpum,   ONLY : calbec_gpu, using_becp_auto, using_becp_d_auto
-  USE becmod_gpum,        ONLY : becp_d
+  USE upf_spinorb,        ONLY : fcoef
   !
   ! Used to avoid unnecessary memcopy
   USE xc_lib,             ONLY : xclib_dft_is
@@ -1000,8 +988,8 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
   INTEGER :: npw, ikb, jkb, ih, jh, ijh, na, np, is, js, nhnt
   ! counters on beta functions, atoms, atom types, spin, and auxiliary vars
   !
-  REAL(DP), POINTER :: becp_d_r_d(:,:)
-  COMPLEX(DP), POINTER :: becp_d_k_d(:,:), becp_d_nc_d(:,:,:)
+  REAL(DP),    ALLOCATABLE :: becp_d_r_d(:,:)
+  COMPLEX(DP), ALLOCATABLE :: becp_d_k_d(:,:), becp_d_nc_d(:,:,:)
 #if defined(__CUDA)
   attributes(DEVICE) :: becp_d_r_d, becp_d_k_d, becp_d_nc_d
 #endif
@@ -1011,23 +999,17 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
   CALL start_clock_gpu( 'sum_band:calbec' )
   npw = ngk(ik)
   IF ( .NOT. real_space ) THEN
-     CALL using_evc_d(0) 
-     CALL using_becp_d_auto(2)
-     ! calbec computes becp = <vkb_i|psi_j>
-!$acc data present(vkb(:,:))
-!$acc host_data use_device(vkb)
-     CALL calbec_gpu( npw, vkb, evc_d(:,ibnd_start:ibnd_end), becp_d )
-!$acc end host_data
-!$acc end data
+     !$acc data present_or_copyin(evc) 
+     CAll calbec(offload_type, npw, vkb, evc(:,ibnd_start:ibnd_end), becp )
+     !$acc end data
   ELSE
-     CALL using_evc(0) 
-     CALL using_becp_auto(2)
      if (gamma_only) then
         do ibnd = ibnd_start, ibnd_end, 2
            call invfft_orbital_gamma(evc,ibnd,ibnd_end) 
            call calbec_rs_gamma(ibnd,ibnd_end,becp%r)
         enddo
         call mp_sum(becp%r,inter_bgrp_comm)
+        !$acc update device(becp%r)
      else
         current_k = ik
         becp%k = (0.d0,0.d0)
@@ -1035,20 +1017,42 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
            call invfft_orbital_k(evc,ibnd,ibnd_end)
            call calbec_rs_k(ibnd,ibnd_end)
         enddo
-       call mp_sum(becp%k,inter_bgrp_comm)
+        call mp_sum(becp%k,inter_bgrp_comm)
+        !$acc update device(becp%k)
      endif
   ENDIF
+  if(allocated(becp%r)) then
+    allocate( becp_d_r_d(size(becp%r,1),size(becp%r,2)) ) 
+    !$acc kernels deviceptr(becp_d_r_d)
+    becp_d_r_d = becp%r 
+    !$acc end kernels
+  elseif(allocated(becp%k)) then
+    allocate( becp_d_k_d(size(becp%k,1),size(becp%k,2)) ) 
+    !$acc kernels deviceptr(becp_d_k_d) 
+    becp_d_k_d = becp%k 
+    !$acc end kernels
+  elseif(allocated(becp%nc)) then
+    allocate( becp_d_nc_d(size(becp%nc,1),size(becp%nc,2),size(becp%nc,3)) ) 
+    !$acc kernels deviceptr(becp_d_nc_d) 
+    becp_d_nc_d = becp%nc 
+    !$acc end kernels
+  endif
   CALL stop_clock_gpu( 'sum_band:calbec' )
   !
   ! In the EXX case with ultrasoft or PAW, a copy of becp will be
   ! saved in a global variable to be rotated later
   IF(xclib_dft_is('hybrid')) THEN       ! This if condition is not present in the CPU code!! Add it?
-     CALL using_becp_auto(0)            ! this is very important to save useless memory copies from GPU to CPU 
+     if(allocated(becp%r)) then
+       !$acc update self(becp%r)
+     elseif(allocated(becp%k)) then
+       !$acc update self(becp%k)
+     elseif(allocated(becp%nc)) then
+       !$acc update self(becp%nc)
+     endif
      CALL store_becxx0(ik, becp)
   ENDIF
   !
   CALL start_clock_gpu( 'sum_band:becsum' )
-  CALL using_becp_d_auto(0)
   !
   DO np = 1, ntyp
      !
@@ -1057,7 +1061,7 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
         ! allocate work space used to perform GEMM operations
         !
         IF ( gamma_only ) THEN
-           nbnd_loc = becp_d%nbnd_loc
+           nbnd_loc = becp%nbnd_loc
            ALLOCATE( auxg_d( nbnd_loc, nh(np) ) )
         ELSE
            ALLOCATE( auxk1_d( ibnd_start:ibnd_end, nh(np)*npol ), &
@@ -1083,7 +1087,6 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
               !
               IF ( noncolin ) THEN
                  !
-                 becp_d_nc_d => becp_d%nc_d
                  !$cuf kernel do(2)
                  DO is = 1, npol
                     DO ih = 1, nhnt
@@ -1103,8 +1106,7 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
                  !
               ELSE IF ( gamma_only ) THEN
                  !
-                 becp_d_r_d => becp_d%r_d
-                 ibnd_begin = becp_d%ibnd_begin
+                 ibnd_begin = becp%ibnd_begin
                  !$cuf kernel do(2)
                  DO ih = 1, nhnt
                     DO ibnd_loc = 1, nbnd_loc
@@ -1114,7 +1116,7 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
                     END DO
                  END DO
                  CALL cublasDgemm ( 'N', 'N', nhnt, nhnt, nbnd_loc, &
-                      1.0_dp, becp_d%r_d(ofsbeta(na)+1,1), nkb,    &
+                      1.0_dp, becp_d_r_d(ofsbeta(na)+1,1), nkb,    &
                       auxg_d, nbnd_loc, 0.0_dp, aux_gk_d, nhnt )
                  !
                  if (tqr) then
@@ -1128,13 +1130,12 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
                    END DO
 
                    CALL cublasDgemm ( 'N', 'N', nhnt, nhnt, nbnd_loc, &
-                        1.0_dp, becp_d%r_d(ofsbeta(na)+1,1), nkb,    &
+                        1.0_dp, becp_d_r_d(ofsbeta(na)+1,1), nkb,    &
                         auxg_d, nbnd_loc, 0.0_dp, aux_egk_d, nhnt )
                  end if
                  !
               ELSE
                  !
-                 becp_d_k_d => becp_d%k_d
                  !$cuf kernel do(2) <<<*,*>>>
                  DO ih = 1, nhnt
                     DO kbnd = 1, this_bgrp_nbnd ! ibnd_start, ibnd_end
@@ -1173,7 +1174,9 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
               IF (noncolin .AND. .NOT. upf(np)%has_so) THEN
                  CALL add_becsum_nc_gpu (na, np, aux_nc_d, becsum_d )
               ELSE IF (noncolin .AND. upf(np)%has_so) THEN
-                 CALL add_becsum_so_gpu (na, np, aux_nc_d, becsum_d )
+!$acc host_data use_device(fcoef)
+                 CALL add_becsum_so_gpu (na, np, fcoef, aux_nc_d, becsum_d )
+!$acc end host_data
               ELSE
                  !
                  !$cuf kernel do(2) <<<*,*>>>
@@ -1219,6 +1222,10 @@ SUBROUTINE sum_bec_gpu ( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd
      !
   END DO
   !
+  if(allocated(becp_d_r_d))  deallocate( becp_d_r_d ) 
+  if(allocated(becp_d_k_d))  deallocate( becp_d_k_d ) 
+  if(allocated(becp_d_nc_d)) deallocate( becp_d_nc_d ) 
+  !
   ! sync 
   if (nhm > 0) then
      becsum=becsum_d
@@ -1290,7 +1297,7 @@ SUBROUTINE add_becsum_nc_gpu ( na, np, becsum_nc_d, becsum_d )
 END SUBROUTINE add_becsum_nc_gpu
 !
 !----------------------------------------------------------------------------
-SUBROUTINE add_becsum_so_gpu( na, np, becsum_nc_d, becsum_d )
+SUBROUTINE add_becsum_so_gpu( na, np, fcoef_d, becsum_nc_d, becsum_d )
   !----------------------------------------------------------------------------
   !! This routine multiplies \(\text{becsum_nc}\) by the identity and the Pauli
   !! matrices, rotates it as appropriate for the spin-orbit case, saves it in 
@@ -1304,10 +1311,12 @@ SUBROUTINE add_becsum_so_gpu( na, np, becsum_nc_d, becsum_d )
   USE uspp_param,           ONLY : nh, nhm
   USE noncollin_module,     ONLY : npol, nspin_mag, domag
   USE uspp,                 ONLY : ijtoh_d, nhtol_d, nhtoj_d, indv_d
-  USE upf_spinorb,          ONLY : fcoef_d
   IMPLICIT NONE
   
   INTEGER, INTENT(IN) :: na, np
+  COMPLEX(DP), INTENT(IN) :: fcoef_d(nhm,nhm,2,2,ntyp)
+  !! function needed to account for spinors.
+!$acc declare deviceptr(fcoef_d)
   COMPLEX(DP), INTENT(IN) :: becsum_nc_d(nh(np),npol,nh(np),npol)
   REAL(DP), INTENT(INOUT) :: becsum_d(nhm*(nhm+1)/2,nat,nspin_mag)
   !
@@ -1315,9 +1324,8 @@ SUBROUTINE add_becsum_so_gpu( na, np, becsum_nc_d, becsum_d )
   !
   INTEGER :: ih, jh, lh, kh, ijh, is1, is2, nhnt
   COMPLEX(DP) :: fac
-
 #if defined(__CUDA)
-  attributes (DEVICE) :: becsum_nc_d, becsum_d
+  attributes (DEVICE) :: fcoef_d, becsum_nc_d, becsum_d
 #endif
   !
   nhnt = nh(np)
