@@ -16,8 +16,8 @@ MODULE uspp_data
   PRIVATE
   !
   PUBLIC :: nqxq, nqx, dq
-  PUBLIC :: qrad,   tab,   tab_at
-  PUBLIC :: qrad_d, tab_d, tab_at_d
+  PUBLIC :: tab,   tab_at
+  PUBLIC :: tab_d
   !
   PUBLIC :: allocate_uspp_data
   PUBLIC :: deallocate_uspp_data
@@ -29,21 +29,17 @@ MODULE uspp_data
   !! number of interpolation points
   REAL(DP), PARAMETER:: dq = 0.01D0
   !! space between points in the pseudopotential tab.
-  REAL(DP), ALLOCATABLE :: qrad(:,:,:,:)
-  !! radial FT of Q functions
   REAL(DP), ALLOCATABLE :: tab(:,:,:)
-  !! interpolation table for PPs
+  !! interpolation table for PP projectorss
   REAL(DP), ALLOCATABLE :: tab_at(:,:,:)
   !! interpolation table for atomic wfc
   !
-  ! GPUs vars
+  !! GPUs variables - only those tables that is useful to have on GPUss
   !
-  REAL(DP), ALLOCATABLE :: qrad_d(:,:,:,:)
   REAL(DP), ALLOCATABLE :: tab_d(:,:,:)
-  REAL(DP), ALLOCATABLE :: tab_at_d(:,:,:)
   !
 #if defined(__CUDA)
-  attributes (DEVICE) :: qrad_d, tab_d, tab_at_d
+  attributes (DEVICE) :: tab_d
 #endif
   !
 contains
@@ -56,30 +52,25 @@ contains
      if (nqxq_/=nqxq) call upf_error("allocate_uspp_data","invalid nqxq_",1)
      if (nqx_/=nqx)   call upf_error("allocate_uspp_data","invalid nqx_",1)
      !
-     if (lmaxq>0) allocate(qrad(nqxq_,nbetam*(nbetam+1)/2, lmaxq, nsp))
      allocate(tab(nqx_,nbetam,nsp))
      allocate(tab_at(nqx_,nwfcm,nsp))
+     !$acc enter data create(tab_at)
      !
      IF (use_gpu) then
         ! allocations with zero size protected
         ! since problematic with CUDAfor
-        if (lmaxq>0.and.nbetam>0)  &
-                       allocate(qrad_d(nqxq_,nbetam*(nbetam+1)/2, lmaxq, nsp))
         if (nbetam>0)  allocate(tab_d(nqx_,nbetam,nsp))
-        if (nwfcm>0)   allocate(tab_at_d(nqx_,nwfcm,nsp))
      endif
      !
   end subroutine allocate_uspp_data
   !
   subroutine deallocate_uspp_data()
      implicit none
-     if( allocated( qrad ) )      deallocate( qrad )
      if( allocated( tab ) )       deallocate( tab )
+     !$acc exit data delete(tab_at)
      if( allocated( tab_at ) )    deallocate( tab_at )
      !
-     if( allocated( qrad_d ) )    deallocate( qrad_d )
      if( allocated( tab_d ) )     deallocate( tab_d )
-     if( allocated( tab_at_d ) )  deallocate( tab_at_d )
   end subroutine
   !
   subroutine scale_uspp_data( vol_ratio_m1 )
@@ -88,13 +79,11 @@ contains
      real(DP), intent(in) :: vol_ratio_m1
      !
      tab(:,:,:)    = tab(:,:,:) * SQRT(vol_ratio_m1)
-     qrad(:,:,:,:) = qrad(:,:,:,:) * vol_ratio_m1
      tab_at(:,:,:) = tab_at(:,:,:) * SQRT(vol_ratio_m1)
 #if defined __CUDA
+!$acc update device (tab_at)
      ! CUDA Fortran safeguard
      if(size(tab) > 0) tab_d = tab
-     if(size(qrad) > 0) qrad_d = qrad
-     if(size(tab_at) > 0) tab_at_d = tab_at
 #endif
   end subroutine scale_uspp_data
   !
